==> mhwaveedit-1.2.3.tar.gz: mhwaveedit-1.2.3/src/samplesizedialog.c <==
/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include "samplesizedialog.h"

static gboolean samplesize_reverse_proc(void *in, guint sample_size,
					chunk_writeout_func out_func, WriteoutID id,
					Chunk *source)
{
     char *c=in, out[4];
     int i;
     for (i=0; i<sample_size; i++) out[i]=c[sample_size-1-i];
     return out_func(id,out,sample_size);
}

static gboolean samplesize_reverse_proc_2(void *in, guint sample_size,
					  chunk_writeout_func out_func, WriteoutID id,
					  Chunk *source)
{
     char *c=in, out[2];
     out[0] = c[1];
     out[1] = c[0];
     return out_func(id,out,2);
}

static Chunk *samplesize_apply_proc(Chunk *chunk, Mainwindow *mw, 
				    gpointer user_data)
{
     Dataformat df;
     SamplesizeDialog *ssd = SAMPLESIZE_DIALOG(user_data);
     Chunk *c,*d;
     if (ssd->gurumode) {
	  d = EFFECT_DIALOG(ssd)->mw->view->chunk;
	  memcpy(&(df),&(d->format),sizeof(Dataformat));
	  df.samplesize = ssd->fs->samplesize;
	  df.sign = ssd->fs->sign;
	  c = chunk_clone_df(d,&df);
     } else {
	  c = chunk_convert_samplesize(chunk,ssd->fs->samplesize,ssd->fs->sign);
     }
     if (!c) return NULL;
     if (ssd->reverse) {
	  if (c->format.samplesize == 2)
	       d = chunk_filter(c,samplesize_reverse_proc_2,NULL,FALSE,FALSE);
	  else
	       d = chunk_filter(c,samplesize_reverse_proc,NULL,FALSE,FALSE);
	       
	  gtk_object_sink(GTK_OBJECT(c));
	  return d;
     } 
     return c;
}


static void samplesize_apply(EffectDialog *ed)
{
     mainwindow_effect_manual(ed->mw,samplesize_apply_proc,FALSE,ed);
     mainwindow_update_texts();
}

static void samplesize_dialog_class_init(EffectDialogClass *klass)
{
     klass->apply = samplesize_apply;
}

static void gurumode_toggle(GtkToggleButton *togglebutton, gpointer user_data)
{
     SamplesizeDialog *ssd = SAMPLESIZE_DIALOG(user_data);
     ssd->gurumode = gtk_toggle_button_get_active(togglebutton);
}

static void reverse_toggle(GtkToggleButton *togglebutton, gpointer user_data)
{
     SamplesizeDialog *ssd = SAMPLESIZE_DIALOG(user_data);
     ssd->reverse = gtk_toggle_button_get_active(togglebutton);
}

static void samplesize_dialog_init(SamplesizeDialog *ssd)
{
     GtkWidget *b,*c;
     GtkBox *ia = GTK_BOX(EFFECT_DIALOG(ssd)->input_area);

     ssd->gurumode = FALSE;
     ssd->reverse = FALSE;

     ssd->fs = FORMAT_SELECTOR(format_selector_new());

     b = gtk_vbox_new(FALSE,3);
     gtk_box_pack_start(ia,b,FALSE,FALSE,0);
     gtk_widget_show(b);
     c = gtk_label_new ( "(This changes the sample type of \n"
			 "the entire file, not just the selection)" );
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);

     c = GTK_WIDGET(ssd->fs);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);

     c=gtk_check_button_new_with_label("Don't actually change the data\n"
				       "(for repairing bad files etc)");
     gtk_signal_connect(GTK_OBJECT(c),"toggled",GTK_SIGNAL_FUNC(gurumode_toggle),
			ssd);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);

     c = gtk_check_button_new_with_label("After converting, reverse byte order "
					 "of samples");
     gtk_signal_connect(GTK_OBJECT(c),"toggled",GTK_SIGNAL_FUNC(reverse_toggle),
			ssd);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);

     gtk_window_set_title(GTK_WINDOW(ssd),"Set sample size");
}

GtkType samplesize_dialog_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "SamplesizeDialog",
	       sizeof(SamplesizeDialog),
	       sizeof(SamplesizeDialogClass),
	       (GtkClassInitFunc) samplesize_dialog_class_init,
	       (GtkObjectInitFunc) samplesize_dialog_init
	  };
	  id = gtk_type_unique(effect_dialog_get_type(),&info);
     }
     return id;
}

GtkWidget *samplesize_dialog_new(Mainwindow *mw)
{
     GtkWidget *w;
     SamplesizeDialog *s;
     w = gtk_type_new(samplesize_dialog_get_type());
     s = SAMPLESIZE_DIALOG(w);
     effect_dialog_set_mainwindow(EFFECT_DIALOG(w),mw);
     format_selector_set(s->fs, mw->view->chunk->format.samplesize, 
			 mw->view->chunk->format.sign);
     return w;
}

==> mhwaveedit-1.3.6.tar.gz: mhwaveedit-1.3.6/src/samplesizedialog.c <==
/*
 * Copyright (C) 2002 2003 2004 2005, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include <config.h>

#include "samplesizedialog.h"
#include "main.h"
#include "effectbrowser.h"

static Chunk *samplesize_apply_proc(Chunk *chunk, StatusBar *bar,
				    gpointer user_data)
{
     Dataformat df;
     SamplesizeDialog *ssd = SAMPLESIZE_DIALOG(user_data);
     memcpy(&(df),&(chunk->format),sizeof(Dataformat));
     format_selector_get(ssd->fs,&df);
     if (ssd->gurumode) 
	  return chunk_clone_df(chunk,&df);
     else
	  return chunk_convert_sampletype(chunk,&df);
}


static gboolean samplesize_apply(EffectDialog *ed)
{
     gboolean b;
     b = mainwindow_effect_manual(EFFECT_BROWSER(ed->eb)->mwl->selected,
				  samplesize_apply_proc,FALSE,ed);
     mainwindow_update_texts();
     return b;
}

static void samplesize_setup(EffectDialog *ed)
{
     Dataformat *f = &(EFFECT_BROWSER(ed->eb)->mwl->format);
     format_selector_set(SAMPLESIZE_DIALOG(ed)->fs, f);
}

static void samplesize_dialog_class_init(EffectDialogClass *klass)
{
     klass->apply = samplesize_apply;
     klass->setup = samplesize_setup;
}

static void gurumode_toggle(GtkToggleButton *togglebutton, gpointer user_data)
{
     SamplesizeDialog *ssd = SAMPLESIZE_DIALOG(user_data);
     ssd->gurumode = gtk_toggle_button_get_active(togglebutton);
}

static void samplesize_dialog_init(SamplesizeDialog *ssd)
{
     GtkWidget *b,*c;
     GtkBox *ia = GTK_BOX(EFFECT_DIALOG(ssd)->input_area);

     ssd->gurumode = FALSE;

     ssd->fs = FORMAT_SELECTOR(format_selector_new(FALSE));

     b = gtk_vbox_new(FALSE,3);
     gtk_box_pack_start(ia,b,FALSE,FALSE,0);
     gtk_widget_show(b);
     c = gtk_label_new ( "(This changes the sample type of "
			 "the entire file, not just the selection)" );
     gtk_label_set_justify(GTK_LABEL(c),GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(c),0,0.5);
     gtk_label_set_line_wrap(GTK_LABEL(c),TRUE);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);

     c = GTK_WIDGET(ssd->fs);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);

     c=gtk_check_button_new_with_label("Don't actually change the data "
				       "(for repairing bad files etc)");
     gtk_signal_connect(GTK_OBJECT(c),"toggled",GTK_SIGNAL_FUNC(gurumode_toggle),
			ssd);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
}

GtkType samplesize_dialog_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "SamplesizeDialog",
	       sizeof(SamplesizeDialog),
	       sizeof(SamplesizeDialogClass),
	       (GtkClassInitFunc) samplesize_dialog_class_init,
	       (GtkObjectInitFunc) samplesize_dialog_init
	  };
	  id = gtk_type_unique(effect_dialog_get_type(),&info);
     }
     return id;
}
